@@ -40,7 +40,6 @@ from hypothesis._settings import (
     Phase,
     Verbosity,
     local_settings,
-    note_deprecation,
     settings as Settings,
 )
 from hypothesis.control import BuildContext
@@ -49,6 +48,7 @@ from hypothesis.errors import (
     DidNotReproduce,
     FailedHealthCheck,
     Flaky,
+    Found,
     HypothesisDeprecationWarning,
     HypothesisWarning,
     InvalidArgument,
@@ -65,12 +65,11 @@ from hypothesis.internal.compat import (
     binary_type,
     get_type_hints,
     getfullargspec,
-    hbytes,
     int_from_bytes,
     qualname,
 )
 from hypothesis.internal.conjecture.data import ConjectureData, StopTest
-from hypothesis.internal.conjecture.engine import ConjectureRunner, ExitReason, sort_key
+from hypothesis.internal.conjecture.engine import ConjectureRunner, sort_key
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.escalation import (
     escalate_hypothesis_internal_error,
@@ -85,10 +84,14 @@ from hypothesis.internal.reflection import (
     get_pretty_function_description,
     impersonate,
     is_mock,
-    nicerepr,
     proxies,
 )
-from hypothesis.reporting import current_verbosity, report, verbose_report
+from hypothesis.reporting import (
+    current_verbosity,
+    report,
+    verbose_report,
+    with_reporter,
+)
 from hypothesis.searchstrategy.collections import TupleStrategy
 from hypothesis.searchstrategy.strategies import MappedSearchStrategy, SearchStrategy
 from hypothesis.statistics import note_engine_for_statistics
@@ -106,10 +109,6 @@ running_under_pytest = False
 global_force_seed = None
 
 
-def new_random():
-    return rnd_module.Random(rnd_module.getrandbits(128))
-
-
 @attr.s()
 class Example(object):
     args = attr.ib()
@@ -227,7 +226,7 @@ class WithRunner(MappedSearchStrategy):
         return "WithRunner(%r, runner=%r)" % (self.mapped_strategy, self.runner)
 
 
-def is_invalid_test(name, original_argspec, generator_arguments, generator_kwargs):
+def is_invalid_test(name, original_argspec, given_arguments, given_kwargs):
     """Check the arguments to ``@given`` for basic usage constraints.
 
     Most errors are not raised immediately; instead we return a dummy test
@@ -243,10 +242,10 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         wrapped_test.is_hypothesis_test = True
         return wrapped_test
 
-    if not (generator_arguments or generator_kwargs):
+    if not (given_arguments or given_kwargs):
         return invalid("given must be called with at least one argument")
 
-    if generator_arguments and any(
+    if given_arguments and any(
         [original_argspec.varargs, original_argspec.varkw, original_argspec.kwonlyargs]
     ):
         return invalid(
@@ -254,32 +253,32 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
             "varkeywords, or keyword-only arguments"
         )
 
-    if len(generator_arguments) > len(original_argspec.args):
-        args = tuple(generator_arguments)
+    if len(given_arguments) > len(original_argspec.args):
+        args = tuple(given_arguments)
         return invalid(
             "Too many positional arguments for %s() were passed to @given "
             "- expected at most %d arguments, but got %d %r"
             % (name, len(original_argspec.args), len(args), args)
         )
 
-    if infer in generator_arguments:
+    if infer in given_arguments:
         return invalid(
             "infer was passed as a positional argument to @given, "
             "but may only be passed as a keyword argument"
         )
 
-    if generator_arguments and generator_kwargs:
+    if given_arguments and given_kwargs:
         return invalid("cannot mix positional and keyword arguments to @given")
     extra_kwargs = [
         k
-        for k in generator_kwargs
+        for k in given_kwargs
         if k not in original_argspec.args + original_argspec.kwonlyargs
     ]
     if extra_kwargs and not original_argspec.varkw:
         arg = extra_kwargs[0]
         return invalid(
             "%s() got an unexpected keyword argument %r, from `%s=%r` in @given"
-            % (name, arg, arg, generator_kwargs[arg])
+            % (name, arg, arg, given_kwargs[arg])
         )
     for a in original_argspec.args:
         if isinstance(a, list):  # pragma: no cover
@@ -289,9 +288,7 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
             )
     if original_argspec.defaults or original_argspec.kwonlydefaults:
         return invalid("Cannot apply @given to a function with defaults.")
-    missing = [
-        repr(kw) for kw in original_argspec.kwonlyargs if kw not in generator_kwargs
-    ]
+    missing = [repr(kw) for kw in original_argspec.kwonlyargs if kw not in given_kwargs]
     if missing:
         return invalid(
             "Missing required kwarg{}: {}".format(
@@ -300,10 +297,37 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         )
 
 
-def execute_explicit_examples(
-    test_runner, test, wrapped_test, settings, arguments, kwargs
-):
-    original_argspec = getfullargspec(test)
+class ArtificialDataForExample(ConjectureData):
+    """Dummy object that pretends to be a ConjectureData object for the purposes of
+    drawing arguments for @example. Provides just enough of the ConjectureData API
+    to allow the test to run. Does not support any sort of interactive drawing,
+    but that's fine because you can't access that when all of your arguments are
+    provided by @example.
+    """
+
+    def __init__(self, kwargs):
+        self.__draws = 0
+        self.__kwargs = kwargs
+
+        def draw_bytes(data, n):
+            raise NotImplementedError()  # pragma: no cover
+
+        super(ArtificialDataForExample, self).__init__(
+            max_length=0, draw_bytes=draw_bytes
+        )
+
+    def draw(self, strategy):
+        assert self.__draws == 0
+        self.__draws += 1
+        # The main strategy for given is always a tuples strategy that returns
+        # first positional arguments then keyword arguments. When building this
+        # object already converted all positional arguments to keyword arguments,
+        # so this is the correct format to return.
+        return (), self.__kwargs
+
+
+def execute_explicit_examples(state, wrapped_test, arguments, kwargs):
+    original_argspec = getfullargspec(state.test)
 
     for example in reversed(getattr(wrapped_test, "hypothesis_explicit_examples", ())):
         example_kwargs = dict(original_argspec.kwonlydefaults or {})
@@ -319,27 +343,37 @@ def execute_explicit_examples(
             )
         else:
             example_kwargs.update(example.kwargs)
-        if Phase.explicit not in settings.phases:
+        if Phase.explicit not in state.settings.phases:
             continue
         example_kwargs.update(kwargs)
-        # Note: Test may mutate arguments and we can't rerun explicit
-        # examples, so we have to calculate the failure message at this
-        # point rather than than later.
-        example_string = "%s(%s)" % (
-            test.__name__,
-            arg_string(test, arguments, example_kwargs),
-        )
-        with local_settings(settings):
+
+        with local_settings(state.settings):
+            fragments_reported = []
+
+            def report_buffered():
+                for f in fragments_reported:
+                    report(f)
+                del fragments_reported[:]
+
             try:
-                with BuildContext(None) as b:
-                    verbose_report("Trying example: " + example_string)
-                    test_runner(None, lambda data: test(*arguments, **example_kwargs))
+                with with_reporter(fragments_reported.append):
+                    state.execute_once(
+                        ArtificialDataForExample(example_kwargs),
+                        is_final=True,
+                        print_example=True,
+                    )
             except BaseException:
-                report("Falsifying example: " + example_string)
-                for n in b.notes:
-                    report(n)
+                report_buffered()
                 raise
 
+            if current_verbosity() >= Verbosity.verbose:
+                prefix = "Falsifying example"
+                assert fragments_reported[0].startswith(prefix)
+                fragments_reported[0] = (
+                    "Trying example" + fragments_reported[0][len(prefix) :]
+                )
+                report_buffered()
+
 
 def get_random_for_wrapped_test(test, wrapped_test):
     settings = wrapped_test._hypothesis_internal_use_settings
@@ -358,7 +392,7 @@ def get_random_for_wrapped_test(test, wrapped_test):
 
 
 def process_arguments_to_given(
-    wrapped_test, arguments, kwargs, generator_kwargs, argspec, test, settings,
+    wrapped_test, arguments, kwargs, given_kwargs, argspec, test, settings,
 ):
     selfy = None
     arguments, kwargs = convert_positional_arguments(wrapped_test, arguments, kwargs)
@@ -386,9 +420,7 @@ def process_arguments_to_given(
     search_strategy = TupleStrategy(
         (
             st.just(arguments),
-            st.fixed_dictionaries(generator_kwargs).map(
-                lambda args: dict(args, **kwargs)
-            ),
+            st.fixed_dictionaries(given_kwargs).map(lambda args: dict(args, **kwargs)),
         )
     )
 
@@ -436,12 +468,10 @@ def failure_exceptions_to_catch():
     return tuple(exceptions)
 
 
-def new_given_argspec(original_argspec, generator_kwargs):
+def new_given_argspec(original_argspec, given_kwargs):
     """Make an updated argspec for the wrapped test."""
-    new_args = [a for a in original_argspec.args if a not in generator_kwargs]
-    new_kwonlyargs = [
-        a for a in original_argspec.kwonlyargs if a not in generator_kwargs
-    ]
+    new_args = [a for a in original_argspec.args if a not in given_kwargs]
+    new_kwonlyargs = [a for a in original_argspec.kwonlyargs if a not in given_kwargs]
     annots = {
         k: v
         for k, v in original_argspec.annotations.items()
@@ -454,7 +484,9 @@ def new_given_argspec(original_argspec, generator_kwargs):
 
 
 class StateForActualGivenExecution(object):
-    def __init__(self, test_runner, search_strategy, test, settings, random, had_seed):
+    def __init__(
+        self, test_runner, search_strategy, test, settings, random, wrapped_test,
+    ):
         self.test_runner = test_runner
         self.search_strategy = search_strategy
         self.settings = settings
@@ -464,7 +496,10 @@ class StateForActualGivenExecution(object):
         self.random = random
         self.__warned_deadline = False
         self.__test_runtime = None
-        self.__had_seed = had_seed
+
+        self.__had_seed = wrapped_test._hypothesis_internal_use_seed
+        self.is_find = getattr(wrapped_test, "_hypothesis_internal_is_find", False)
+        self.wrapped_test = wrapped_test
 
         self.test = test
 
@@ -484,6 +519,9 @@ class StateForActualGivenExecution(object):
         it might have placed ``data`` in an unsuccessful state and then
         swallowed the corresponding control exception.
         """
+
+        data.is_find = self.is_find
+
         text_repr = [None]
         if self.settings.deadline is None:
             test = self.test
@@ -639,10 +677,14 @@ class StateForActualGivenExecution(object):
         """
         # Tell pytest to omit the body of this function from tracebacks
         __tracebackhide__ = True
-        if global_force_seed is None:
-            database_key = function_digest(self.test)
-        else:
-            database_key = None
+        try:
+            database_key = self.wrapped_test._hypothesis_internal_database_key
+        except AttributeError:
+            if global_force_seed is None:
+                database_key = function_digest(self.test)
+            else:
+                database_key = None
+
         runner = ConjectureRunner(
             self._execute_once_for_engine,
             settings=self.settings,
@@ -691,7 +733,7 @@ class StateForActualGivenExecution(object):
             try:
                 self.execute_once(
                     ran_example,
-                    print_example=True,
+                    print_example=not self.is_find,
                     is_final=True,
                     expected_failure=(
                         info.__expected_exception,
@@ -767,7 +809,7 @@ def fake_subTest(self, msg=None, **__):
     """Monkeypatch for `unittest.TestCase.subTest` during `@given`.
 
     If we don't patch this out, each failing example is reported as a
-    seperate failing test by the unittest test runner, which is
+    separate failing test by the unittest test runner, which is
     obviously incorrect. We therefore replace it for the duration with
     this version.
     """
@@ -801,8 +843,8 @@ class HypothesisHandle(object):
 
 
 def given(
-    *given_arguments,  # type: Union[SearchStrategy, InferType]
-    **given_kwargs  # type: Union[SearchStrategy, InferType]
+    *_given_arguments,  # type: Union[SearchStrategy, InferType]
+    **_given_kwargs  # type: Union[SearchStrategy, InferType]
 ):
     # type: (...) -> Callable[[Callable[..., None]], Callable[..., None]]
     """A decorator for turning a test function that accepts arguments into a
@@ -811,18 +853,18 @@ def given(
     This is the main entry point to Hypothesis.
     """
 
-    def run_test_with_generator(test):
+    def run_test_as_given(test):
         if inspect.isclass(test):
             # Provide a meaningful error to users, instead of exceptions from
             # internals that assume we're dealing with a function.
             raise InvalidArgument("@given cannot be applied to a class.")
-        generator_arguments = tuple(given_arguments)
-        generator_kwargs = dict(given_kwargs)
+        given_arguments = tuple(_given_arguments)
+        given_kwargs = dict(_given_kwargs)
 
         original_argspec = getfullargspec(test)
 
         check_invalid = is_invalid_test(
-            test.__name__, original_argspec, generator_arguments, generator_kwargs
+            test.__name__, original_argspec, given_arguments, given_kwargs
         )
 
         # If the argument check found problems, return a dummy test function
@@ -832,16 +874,16 @@ def given(
 
         # Because the argument check succeeded, we can convert @given's
         # positional arguments into keyword arguments for simplicity.
-        if generator_arguments:
-            assert not generator_kwargs
+        if given_arguments:
+            assert not given_kwargs
             for name, strategy in zip(
-                reversed(original_argspec.args), reversed(generator_arguments)
+                reversed(original_argspec.args), reversed(given_arguments)
             ):
-                generator_kwargs[name] = strategy
+                given_kwargs[name] = strategy
         # These have been converted, so delete them to prevent accidental use.
-        del generator_arguments
+        del given_arguments
 
-        argspec = new_given_argspec(original_argspec, generator_kwargs)
+        argspec = new_given_argspec(original_argspec, given_kwargs)
 
         @impersonate(test)
         @define_function_signature(test.__name__, test.__doc__, argspec)
@@ -870,26 +912,20 @@ def given(
 
             # Use type information to convert "infer" arguments into appropriate
             # strategies.
-            if infer in generator_kwargs.values():
+            if infer in given_kwargs.values():
                 hints = get_type_hints(test)
             for name in [
-                name for name, value in generator_kwargs.items() if value is infer
+                name for name, value in given_kwargs.items() if value is infer
             ]:
                 if name not in hints:
                     raise InvalidArgument(
                         "passed %s=infer for %s, but %s has no type annotation"
                         % (name, test.__name__, name)
                     )
-                generator_kwargs[name] = st.from_type(hints[name])
+                given_kwargs[name] = st.from_type(hints[name])
 
             processed_args = process_arguments_to_given(
-                wrapped_test,
-                arguments,
-                kwargs,
-                generator_kwargs,
-                argspec,
-                test,
-                settings,
+                wrapped_test, arguments, kwargs, given_kwargs, argspec, test, settings,
             )
             arguments, kwargs, test_runner, search_strategy = processed_args
 
@@ -912,12 +948,7 @@ def given(
                 )
 
             state = StateForActualGivenExecution(
-                test_runner,
-                search_strategy,
-                test,
-                settings,
-                random,
-                had_seed=wrapped_test._hypothesis_internal_use_seed,
+                test_runner, search_strategy, test, settings, random, wrapped_test,
             )
 
             reproduce_failure = wrapped_test._hypothesis_internal_use_reproduce_failure
@@ -963,9 +994,7 @@ def given(
             # There was no @reproduce_failure, so start by running any explicit
             # examples from @example decorators.
 
-            execute_explicit_examples(
-                test_runner, test, wrapped_test, settings, arguments, kwargs
-            )
+            execute_explicit_examples(state, wrapped_test, arguments, kwargs)
 
             # If there were any explicit examples, they all ran successfully.
             # The next step is to use the Conjecture engine to run the test on
@@ -1024,7 +1053,7 @@ def given(
                         raise
                     # On Python 3, we swap out the real traceback for our
                     # trimmed version.  Using a variable ensures that the line
-                    # which will actually appear in trackbacks is as clear as
+                    # which will actually appear in tracebacks is as clear as
                     # possible - "raise the_error_hypothesis_found".
                     the_error_hypothesis_found = e.with_traceback(
                         get_trimmed_traceback()
@@ -1053,7 +1082,7 @@ def given(
         wrapped_test.hypothesis = HypothesisHandle(test)
         return wrapped_test
 
-    return run_test_with_generator
+    return run_test_as_given
 
 
 def find(
@@ -1066,14 +1095,11 @@ def find(
     # type: (...) -> Any
     """Returns the minimal example from the given strategy ``specifier`` that
     matches the predicate function ``condition``."""
-    note_deprecation(
-        "`find(s, f)` is deprecated, because it is rarely used but takes "
-        "ongoing work to maintain as we upgrade other parts of Hypothesis.",
-        since="2019-07-11",
-    )
     if settings is None:
         settings = Settings(max_examples=2000)
-    settings = Settings(settings, suppress_health_check=HealthCheck.all())
+    settings = Settings(
+        settings, suppress_health_check=HealthCheck.all(), report_multiple_bugs=False
+    )
 
     if database_key is None and settings.database is not None:
         database_key = function_digest(condition)
@@ -1085,60 +1111,24 @@ def find(
         )
     specifier.validate()
 
-    search = specifier
+    last = [None]
 
-    random = random or new_random()
-    successful_examples = [0]
-    last_data = [None]
-    last_repr = [None]
+    @settings
+    @given(specifier)
+    def test(v):
+        if condition(v):
+            last[0] = v
+            raise Found()
 
-    def template_condition(data):
-        with deterministic_PRNG():
-            with BuildContext(data):
-                try:
-                    data.is_find = True
-                    result = data.draw(search)
-                    data.note(result)
-                    success = condition(result)
-                except UnsatisfiedAssumption:
-                    data.mark_invalid()
+    if random is not None:
+        test = seed(random.getrandbits(64))(test)
 
-        if success:
-            successful_examples[0] += 1
+    test._hypothesis_internal_is_find = True
+    test._hypothesis_internal_database_key = database_key
 
-        if settings.verbosity >= Verbosity.verbose:
-            if not successful_examples[0]:
-                report(u"Tried non-satisfying example %s" % (nicerepr(result),))
-            elif success:
-                if successful_examples[0] == 1:
-                    last_repr[0] = nicerepr(result)
-                    report(u"Found satisfying example %s" % (last_repr[0],))
-                    last_data[0] = data
-                elif (
-                    sort_key(hbytes(data.buffer)) < sort_key(last_data[0].buffer)
-                ) and nicerepr(result) != last_repr[0]:
-                    last_repr[0] = nicerepr(result)
-                    report(u"Shrunk example to %s" % (last_repr[0],))
-                    last_data[0] = data
-        if success and not data.frozen:
-            data.mark_interesting()
-
-    runner = ConjectureRunner(
-        template_condition, settings=settings, random=random, database_key=database_key
-    )
-    runner.run()
-    note_engine_for_statistics(runner)
-    if runner.interesting_examples:
-        data = ConjectureData.for_buffer(
-            list(runner.interesting_examples.values())[0].buffer
-        )
-        with deterministic_PRNG():
-            with BuildContext(data):
-                return data.draw(search)
-    if runner.valid_examples == 0 and (runner.exit_reason != ExitReason.finished):
-        raise Unsatisfiable(
-            "Unable to satisfy assumptions of %s."
-            % (get_pretty_function_description(condition),)
-        )
+    try:
+        test()
+    except Found:
+        return last[0]
 
     raise NoSuchExample(get_pretty_function_description(condition))
